/* client/Widgets.h

Project : Arianne RPG
Created : 18 Aug 2000

Common GUI Widgets.

*/

#ifndef __ARIANNE_RPG_WIDGETS_H
#define __ARIANNE_RPG_WIDGETS_H
#ifndef __cplusplus
#error C++ compiler required.
#endif

#include "../common/general.h"
#include "../common/astring.h"
#include "../common/List.h"

// Declare in advance as we'll need it in AWidget
class WidgetGroup;

// Abstract Widget class
class AWidget : public Disposable {
protected:
    WidgetGroup *parent;
public:
    AWidget(void) { parent=NULL; };
    AWidget(WidgetGroup *myparent) { parent=myparent; };
    ~AWidget(void);
    // Keep everything running
    void virtual Run(void);
    // Check if a point lies in widget
    BOOL virtual IsPointIn(Sint16 x, Sint16 y) = 0;
    // Move the widget relative to the old position
    void virtual Move(Sint16 dx, Sint16 dy) = 0;
    // (re)Paint the widget
    void virtual Paint(SDL_Surface *surface) = 0;
    // Handle an event, by default ignore all
    BOOL virtual HandleEvent(SDL_Event *event) { return FALSE; };
    // Get the parent widget
    WidgetGroup *GetParent(void) { return parent; };
    // Set a new parent widget, return the old one
    WidgetGroup *SetParent(WidgetGroup *newparent);
};

// A group of widgets
class WidgetGroup : public AWidget {
protected:
    int timeout;
    LIST childs;
public:
    // Special visibility timeouts
    enum { VISIBLE=-1, INVISIBLE=0 };
    // Check if the group is currently visible
    BOOL IsVisible(void) { return (timeout != INVISIBLE); };
    // Set a visibility timeout
    void SetVisibleTime(int time);

    void virtual Run(void);
    BOOL virtual IsPointIn(Sint16 x, Sint16 y);
    void virtual Move(Sint16 dx, Sint16 dy);
    void virtual Paint(SDL_Surface *surface);
    BOOL virtual HandleEvent(SDL_Event *event);
};

// A pixmapped splash area
class Splash : public AWidget {
};

// A floating text
class FloatText : public AWidget {
protected:
    AString textvalue;
public:
    // Get the text
    const AString &GetText(void) { return textvalue; };
    // Set a new text
    void virtual SetText(const char *newtext);

    void virtual Paint(SDL_Surface *surface);
};

// An interactive text editor
class TextEdit : public FloatText {
protected:
    BOOL insmode;
    int style;
    int position;
public:
    // Cursor styles
    enum { NONE=0, THIN=1, UNDER=2, BLOCK=3 };
    // Get cursor style
    int GetCursorStyle(void) { return style; };
    // Set a new cursor style, return old one
    int virtual SetCursorStyle(int newstyle);
    // Get cursor position
    int GetCursorPos(void) { return position; };
    // Set cursor position in text, return old pos
    int virtual SetCursorPos(int newpos);

    void virtual SetText(const char *newtext);
    void virtual Paint(SDL_Surface *surface);
    BOOL virtual HandleEvent(SDL_Event *event);
};

// Abstract button class
class Button : public AWidget {
protected:
    int status;
public:
    // Button states
    enum { UP=0, DOWN=1, HOVER=2, GRAY=3 };
    // Get current button state
    int GetButtonState(void) { return status; };
    // Set new state, returns old
    int virtual SetButtonState(int newstate);

    BOOL virtual HandleEvent(SDL_Event *event);
};

// Rectangular text button
class RectButton : public Button {
protected:
    AString caption;
public:
    // Get the text
    const AString &GetText(void) { return caption; };
    // Set a new text
    void virtual SetText(const char *newtext);

    void virtual Paint(SDL_Surface *surface);
};

// Pixmapped button
class PixButton : public Button {
};

#endif /* __ARIANNE_RPG_WIDGETS_H */
