#ifndef ARIANNE_RPG_AList_H
#define ARIANNE_RPG_AList_H

#include "general.h"
#include "templock.h"
#include "mutex.h"

template <class BaseType>
class AList
  {
  BaseType **list;
  Uint32 NumCells,NumMaxCells;
  Mutex m_mutex;

  public:

  AList(Uint32 size=16)
    {
    TempLock lock(m_mutex);

    NumMaxCells=size;
    list=new BaseType*[NumMaxCells];
    Clear();
    }

  ~AList() 
    {
    TempLock lock(m_mutex);

    Delete();
    delete[] list;
    list=0;
    NumMaxCells=0;
    NumCells=0;
    }

  /* This empty the list WITHOUT freeing the memory of the objects. */
  void Clear()
    {
    TempLock lock(m_mutex);

    NumCells=0;
    for(Uint32 i=0;i<NumMaxCells;i++)
      {
      list[i]=0;
      }
    }

  /* This empty the list AND free the memory of the objects. */
  void Delete()
    {
    TempLock lock(m_mutex);

    NumCells=0;
    for(Uint32 i=0;i<NumMaxCells;i++)
      {
      if(list[i])
        {
        delete list[i];
        list[i]=0;
        }
      }
    }

  /** Returns TRUE if the list is empty */
  BOOL isEmpty()
    {
    TempLock lock(m_mutex);

    if(NumCells==0) return TRUE;
    return FALSE;
    }

  /** This Add a new object to the List. Pos is useless by now */
  void Add(BaseType *object,Uint32 pos=0)
    {
    TempLock lock(m_mutex);

    assert(object!=NULL);

    /* Here come one of the tricky parts, we do an exponential grow of the buffer if needed. */
    if(NumCells<NumMaxCells)
      {
      list[NumCells]=object;
      NumCells++;
      return;
      }
    else
      {
      if(NumMaxCells==MAX_ALIST_SIZE)
        {
        throw "AList is full (65536).\n";
        }
      else
        {
        NumMaxCells=NumMaxCells<<1;
        BaseType **newlist=new BaseType*[NumMaxCells];

        for(Uint32 i=0;i<NumCells;i++)
          {
          newlist[i]=list[i];
          }
        newlist[NumCells]=object;
        NumCells++;
      
        delete[] list;
        list=newlist;
        }
      }
    }

  /** This delete the object of the given pos */
  void Free(Uint32 pos)
    {
    TempLock lock(m_mutex);

    assert(pos<NumCells);
    assert(list[pos]!=NULL);

    delete list[pos];

    /* We have now two possibilities, keep order or not */
    #ifdef KEEP_SORT_LIST
      /* Keeping */
      for(Uint32 i=pos+1;i<NumCells;i++)
        {
        list[i-1]=list[i];
        }
      list[NumCells]=0;
      NumCells--;
    #else
      /* Not keeping */
      list[pos]=list[NumCells];
      NumCells--;
    #endif            

    return;
    }
 
  /** This remove the object from the given pos and return it */
  BaseType *Remove(Uint32 pos)
    {
    TempLock lock(m_mutex);

    assert(pos<NumCells);
    assert(list[pos]!=NULL);

    BaseType *tmp=0;
    tmp=list[pos];

    /* We have now two possibilities, keep order or not */
    #ifdef KEEP_SORT_LIST
      /* Keeping */
      for(Uint32 i=pos+1;i<NumCells;i++)
        {
        list[i-1]=list[i];
        }
      list[NumCells]=0;
      NumCells--;
    #else
      /* Not keeping */
      list[pos]=list[NumCells];
      NumCells--;
    #endif       
   
    return tmp;     
    }

  /** Return the number of objects that are on list */
  Uint32 Size()
    {
    TempLock lock(m_mutex);

    return NumCells;
    }

  /** An operator to access the list. */
  BaseType *operator[](Uint32 i)
    {
    TempLock lock(m_mutex);

    assert(i<NumCells);

    return list[i];
    }

  int operator+(AList<BaseType> *listb)
    {
    TempLock lock(m_mutex);

    assert(listb!=NULL);

    for(Uint32 i=0;i<listb->Size();i++)
      {
      Add((*listb)[i]);
      }

    return listb->Size();
    }


  /* Here come the best part */
  /** Returns the size in bytes of the list */
  Uint32 GetSize()
    {
    TempLock lock(m_mutex);

    Uint32 lsize=0;

    for(Uint32 i=0;i<NumCells;i++)
      {
      lsize+=list[i]->GetSize();
      }

    return lsize+sizeof(Uint32) /* Because of the size */;
    } 

  /** Convert the List to a vector. */
  void ToVector(AVector *Vector)
    {
    /* NOTE: When the size of the list grows, the dump is very slow, so splitting
       the dump in smaller blocks and joining them later is faster.( VERY FASTER ) */

    assert(Vector!=NULL);

    TempLock lock(m_mutex);

    AUint32 lsize=Size();
    lsize.ToVector(Vector);

    /* How many blocks ? */
	Uint32 NumVect=NumCells/OBJECT_BUFFER;
	if(NumCells%OBJECT_BUFFER!=0)
	  {
	  NumVect++;
	  }

    Uint32 tmpNumCells=NumCells;    

    /* Create the vectors */
    AList<AVector> *Vectors=new AList<AVector>;
    Uint32 i;

    for(i=0;i<NumVect;i++)
      {
      Vectors->Add(new AVector());
      }

    /* Start to dump it */
    for(i=0;i<NumVect;i++)
      {
      Uint32 Data2Process=(OBJECT_BUFFER<tmpNumCells)?OBJECT_BUFFER:tmpNumCells;
      for(Uint32 j=0;j<Data2Process;j++)
        {
        list[i*OBJECT_BUFFER+j]->ToVector((*Vectors)[i]);
        } 

      tmpNumCells-=Data2Process;
      }

    /* Now we join it */
    for(i=0;i<NumVect;i++)
      {      
      (*Vector)+((*Vectors)[i]);
      }

    /* Free memory */
    delete Vectors;
    }

  /** Convert the list from a vector */
  void FromVector(AVector* data)
    {
    TempLock lock(m_mutex);

    assert(data!=NULL);

    AUint32 lsize;
    lsize.FromVector(data);

    this->Delete();

    for(Uint32 i=0;i<(Uint32)lsize;i++)
      {
      BaseType *tmp=new BaseType;
      tmp->FromVector(data);
 
      this->Add(tmp);
      }
    }
  };

#endif
