#ifndef ARIANNE_Callback_h
#define ARIANNE_Callback_h

/*
        (c) Rosimildo da Silva -- rdasilva@connecttel.com
        Under the LGPL
*/


#include "general.h"

/////////////////////////////////////////////////////////////////////////////
//  
// DESCRIPTION:
//
// This is an abstract class that defines the interface for the Callback
// framework for C++. It is templated on the message "Parameter" to be
// used by the parties involved.
//
/////////////////////////////////////////////////////////////////////////////
template< class Parameter > 
class Callback
{
public:
   // Contract between the clients and the subsystems
   virtual int notify( Parameter * ) = 0;

   virtual ~Callback() { }

   virtual bool operator==( const Callback& c ) { return( this == &c); }
   virtual bool operator<( const Callback &  )  { return 1; }

   // Creates a copy of the instance...
   virtual Callback< Parameter > *clone() = 0;

protected:
   Callback(){}
   // Copy and assignment are not public at this level...
   Callback( const Callback & ){}
   Callback & operator = ( const Callback & ){}
};




/////////////////////////////////////////////////////////////////////////////
//  
// DESCRIPTION:
//
// This is a concrete class that implements the interface for the Callback
// framework for C++ for Global functions. This class should be used with
// global functions or static methods of a particular class.
//
/////////////////////////////////////////////////////////////////////////////
template < class Parameter >
class FunctionCallback : public Callback< Parameter >
{
public:
   FunctionCallback() : funcPtr( 0 ) { }
   FunctionCallback( int ( *fPtr ) ( Parameter * ) ) : funcPtr( fPtr ) { }
   virtual ~FunctionCallback() { }

   virtual bool operator==( const FunctionCallback & c ) { return( funcPtr == c.funcPtr ); }

   virtual int notify( Parameter *param ) { return funcPtr ? funcPtr( param ) : 0; }
   void setFunction( int (*fPtr) (Parameter *) )  { funcPtr = fPtr; }

   // Make a copy of this object
   Callback< Parameter > *clone() 
   { return new FunctionCallback< Parameter >( *this ); }

private:
   int ( *funcPtr )( Parameter * );
};





/////////////////////////////////////////////////////////////////////////////
//  
// DESCRIPTION:
//
// This is a concrete class that implements the interface for the Callback
// framework for C++ for member functions of a class. This class should be 
// used with methods of a class.
//
/////////////////////////////////////////////////////////////////////////////
template < class Parameter, class ClassType >
class MemberCallback : public Callback< Parameter >
{
public:
   MemberCallback() : memberPtr( 0 ), instance( 0 ) { }
   MemberCallback( int (ClassType::*f) (Parameter *), ClassType* o ) :
     memberPtr( f ), instance( o ) {}
   virtual ~MemberCallback() { }

   virtual bool operator==( const MemberCallback& c ) 
   { 
     return( ( memberPtr == c.memberPtr ) && ( instance == c.instance ) ); 
   }

   virtual int notify( Parameter* param )  
   { return  instance ? ( instance->*memberPtr )( param ) : 0; }
   void setMember( int (ClassType::*f) (Parameter* ), ClassType* o )
   {  memberPtr= f;  instance = o;  }

   // Make a copy of this object
   Callback< Parameter > *clone() 
   { return new MemberCallback< Parameter, ClassType >( *this ); }

private:
   int (ClassType::*memberPtr) (Parameter *);
   ClassType  *instance;
};


#endif