/* common/inifile.c

Project : Arianne RPG
Created : 27 Jun 2000

Implement basic Windows-style profile files.

*/

#if defined(__linux__) || defined(__beos__)// || defined(__macos__)

#include "inifile.h"
#include <ctype.h>
#include <stdio.h>
#include <sys/stat.h>
#include <string.h>

#define BUFSIZE 2048

static void trimExt(char *buf)
{
    char *p;
    for (p=buf;*buf;buf++)
	if (*buf=='/' || *buf=='\\') p=buf+1;
    while (*p && *p!='.') p++;
    *p=NUL;
}

static void uniformize(const char *src, char *dest, int size)
{
/* should we replace size with sizeof(src) here ? */
    BOOL hasspace;
    if (size < 0 || dest==NULL) return;
    if (src) {
        // skip leading spaces
        while (isspace(*src)) src++;

        while (*src && size) {
	        hasspace=FALSE;
	        while (isspace(*src)) {
		        hasspace=TRUE;
		        src++;
	        }
	        if (*src && !isspace(*src)) {
		        if (hasspace && (size > 1)) {
		            *dest++=' ';
		            size--;
        		}
		        *dest++=tolower(*src++);
        		size--;
	        }
	    }
    }
    *dest=NUL;
}

static BOOL isDiskFile(FILE *f)
{
    struct stat statbuf;
    return (fstat(f->_fileno,&statbuf)==0) && S_ISREG(statbuf.st_mode);
}

/*  Retrive a character string from the ini file in the buffer.
    If key==NULL it returns a double NUL terminated list of keys.
    The default value must never be NULL.
    Section and key names are case insensitive.
    Returns the number of characters placed in the buffer
*/
size_t GetIniFileStr(const char *section, const char *key, const char *defval,
	char *buffer, size_t buflen, const char *inifile)
{
    FILE* f;
    char buf1[BUFSIZE],buf2[BUFSIZE],buf3[BUFSIZE],*p;
    BOOL found,notfound,inblock;
    int copied=0;
    found=FALSE;
    uniformize(inifile,buf1,sizeof(buf1)-1);

    if (f=fopen(buf1,"r")) {
        if (isDiskFile(f)) {
            notfound=inblock=FALSE;
            uniformize(section,buf1,sizeof(buf1)-1);
            while (!(found || notfound || feof(f))) {
	            fgets(buf2,sizeof(buf2),f);
	            p=strchr(buf2,'\n');
	            if (p) *p=NUL;
	            else if (*buf2==NUL) break;
	            uniformize(buf2,buf3,sizeof(buf3)-1);
	            if (!notfound) {
	                if (inblock) {
                        if (*buf3=='[') {
    		                inblock=FALSE;
	    	                notfound=TRUE;
	                    } else {
		                    if (p=strchr(buf2,'=')) {
			                    *p=NUL;
			                    if (key) {
				                    uniformize(buf2,buf3,sizeof(buf3)-1);
				                    if (!strcmp(buf1,buf3)) {
					                    found=TRUE;
    					                do { p++; } while isspace(*p);
    				                }
	    	    		        } else {
		    		                size_t n=strlen(++p)+1;
			    	                if (n > buflen-1) n=buflen-1;
				                    if (n > 0) {
					                    memcpy(buffer,p,n);
				                        copied+=n;
					                    buflen-=n;
				                    }
				                }
			                }
			            }
		            } else if (*buf3=='[') {
			            if (p=strchr(buf3+1,']')) {
			                *p=NUL;
			                uniformize(buf3+1,buf2,sizeof(buf2)-1);
			                if (!strcmp(buf1,buf2)) {
				                inblock=TRUE;
				                if (key) uniformize(key,buf1,sizeof(buf1)-1);
			                }
			            }
		            }
		        }
	        }
	    }
    	fclose(f);
    }

    if (key==NULL) {
	    buffer[copied++]=NUL;
    	return copied;
    }
    if (found) strncpy(buffer,p,buflen);
    else strncpy(buffer,defval,buflen);
    return strlen(buffer);
}

/*  Store a character string in the ini file.
    If value==NULL it deletes the entire key.
    If key==NULL it deletes the entire section.
    Returns TRUE is successfull.
*/   
BOOL PutIniFileStr(const char *section, const char *key, const char *value,
	const char *inifile)
{
    FILE *fi,*fo;
    char buf1[BUFSIZE],buf2[BUFSIZE],buf3[BUFSIZE],buf4[BUFSIZE];
    char bufo[BUFSIZE],bufn[BUFSIZE],*p;
    BOOL fileok,found,inblock,insect;
    int extral,maxtry;
    found=fileok=FALSE;
    uniformize(inifile,bufo,sizeof(bufo)-1);
    if (fi=fopen(bufo,"rt")) {
	    if (isDiskFile(fi)) {
    		strcpy(bufn,bufo);
	    	for (maxtry=100;maxtry;maxtry--) {
		        trimExt(bufn);
		        sprintf(strchr(bufn,NUL),".~%02d",rand()%100);
    		    if (access(bufn,0)) {
	        		fo=fopen(bufn,"wt");
			        break;
    		    }
	    	}
	    	
		    if (fo) {
		        fileok=TRUE;
    		    insect=FALSE;
	    	    extral=0;
		        uniformize(section,buf1,sizeof(buf1)-1);
		        while (fileok && !feof(fi)) {
    	    		*buf2=NUL;
	    	    	fgets(buf2,sizeof(buf2),fi);
        			p=strchr(buf2,'\n');
		        	if (p) *p=NUL;
        			else if (*buf2==NUL) break;
		        	uniformize(buf2,buf3,sizeof(buf3)-1);
        			if (*buf3=='[') {
		        	    if (p=strchr(buf3+1,']')) {
				            *p=NUL;
            				uniformize(buf3+1,buf4,sizeof(buf4)-1);
			            	inblock=(strcmp(buf1,buf4)==0);
            				if (insect && !found) {
			            	    found=TRUE;
            				    if (value) fprintf(fo,"%s=%s\n",key,value);
			            	}
            				for (;extral;extral--) fprintf(fo,"\n");
		                	insect=inblock;
			            }
        			    inblock=TRUE;
        			} else inblock=FALSE;
	            	if (insect) {
		                if (key) {
                   			if (inblock) p=NULL;
               				else {
   		            		    p=strchr(buf3,';');
   				                if (p==buf3) p=NULL;
           	    			    else p=strchr(buf3,'=');
	                		}
           			    	if (p) {
		            	        *p=NUL;
			                    uniformize(key,buf4,sizeof(buf4)-1);
               				    if (strcmp(buf3,buf4)==0) {
                    				found=TRUE;
		        	            	p=strchr(buf2,'=');
           					        if (value && p) {
       	            				    p[1]=NUL;
	        		            	    fprintf(fo,"%s%s\n",buf2,value);
           				        	}
   		            		    } else p=NULL;
           	    			}
	                		if (p==NULL) {
		    	                if (*buf2==NUL) extral++;
           				        else {
		            		        for (;extral;extral--) fprintf(fo,"\n");
       				            	fprintf(fo,"%s\n",buf2);
           		    		    }
		                	}
           			    } else {
   		            		extral=0;
           	    			found=TRUE;
		                }
           			} else fprintf(fo,"%s\n",buf2);
           	    }
           	    if (key && !found) {
		            if (!insect) fprintf(fo,"[%s]\n",section);
           			if (value) fprintf(fo,"%s=%s\n",key,value);
		            for (;extral;extral--) fprintf(fo,"\n");
           	    }
           	    fclose(fo);
           	    if (!fileok) unlink(bufn);
           	}
        }
        fclose(fi);
        if (fileok) {
#if defined(_Windows) || defined(__WIN32__)
            unlink(bufo);
#endif
       	    rename(bufn,bufo);
       	}
    }
    else if (fo=fopen(bufo,"wt")) {
       	if (isDiskFile(fo)) {
            if (key) {
           		fprintf(fo,"[%s]\n",section);
   	        	if (value) fprintf(fo,"%s=%s\n",key,value);
       	    	fprintf(fo,"\n");
            }
       	    fileok=TRUE;
       	}
      	fclose(fo);
    }
    return fileok;
}

#endif