/***************************************************************************
                          thread.h  -  description
                             -------------------
    begin                : Tue Jul 4 2000
    copyright            : (C) 2000 by Ulrich Eckhardt
    email                : Doomster@knuut.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _ARIANNE_THREAD_H
#define _ARIANNE_THREAD_H

/**Encapsulation of the SDL-library threading facilities:

   Danger: The constructor runs in a different thread than the Thread-object itself,
   do not asssume things like threadID() to work properly right after creating the object!

   The auto-delete function is to just fire up one object and stop caring about it. Possible
   applications are SFX-player threads: just sdtart playing a sound now and forget about when it ends.
   Other thing is the auto-delete function, Im not so sure if this is a)necessary and b)properly
   implemented, YMMV - lets hear it.
		
   To create a thread for some task, derive a class from Thread and supply the thread-function.
    If your Thread can run as it is created (if all necessary data are give to the constructor),
   you can call start() in the constructor.
    Else, make all necessary preparations and then start() the thread.

  *@author Ulrich Eckhardt
  */

#include "general.h"
#include "mutex.h"
#include <SDL/SDL_thread.h>

/** This is the global thread-function that is entered by every SDL_thread. It is
always given the address of a Thread-object as argument and simply calls the
objects threadFunction().
It has to be declared with C-linkage since it is acessed by a C-library. */
extern "C" int commonThreadFunction(void*);


class Thread
{
public: 
	Thread();
	virtual ~Thread();

	void start();
	/** Get the 32-bit thread identifier for the thread */
	Uint32 threadID() const;

	/** By setting the auto-delete flag you can create threads with new that will
	automagically delete themselves after they finished their tasks.
	Dont use this if you need the results of the threads work since it will delete
	itself when finished and you cant use the pointer/reference anymore after that
	without invoking undefined behaviour. */
	void setAutoDelete(bool flag);

	/** returns the status of the auto-delete flag */
	bool autoDelete() const;

	/** Wait for a thread to finish. The return code for the thread function is placed
	in the area pointed to by 'status', if 'status' is not null.*/
	virtual void waitThread(int *status = 0);

	/** Forcefully kill a thread without worrying about its state(welcome to Memory,Leak&Co Ltd) */
	virtual void killThread();

	// Minami's addition
#ifdef macintosh
	virtual int	threadFunction_Mac()=0;
#endif
	// end of addition
			
protected:
	/** This is the thread function that is called by the framework. Override this to
	give the class some functionality. The default implementation is pretty useless,
	it merely serves debugging purposes. */
	virtual int threadFunction()=0;

private:
	SDL_Thread* m_SDLThread;
	Uint32 m_ID;
	bool m_autoDelete;
	
	// Minami's addition
	#ifdef macintosh
	ThreadID	m_threadID_Mac;
	#endif
	// end of addition

	/** do not use */
	Thread(Thread const&);

	/** do not use */
	Thread const& operator=(Thread const&);

	/** we have to declare this a friend since it accesses the Thread-objects private parts */	
	friend int commonThreadFunction(void*);
};	


/** This class features a thread that perpetually call its loop() function.
Furthermore, this thread doesnt start acting until you specifically tell it
to and features a stop()-method that resembles the waitThread()-method but 
asks the thread to terminate via IPC(a simple bool-variable). */

class LoopThread:
	private Thread
{
public:
	LoopThread();
	~LoopThread();

	/** the thread is created in suspended state, this method starts it*/ 
	virtual void start();
	/** this method suspends the thread after the current loop()-call
	returns */
	virtual void suspend();
	/** this method terminates the thread after the current loop()-call
	returns */
	virtual int stop();
	// I dont know why this doesnt work
	// public: Uint32 Thread::threadID() const;
	Uint32 threadID() const
	{return Thread::threadID();}
    /** returns true if the Thread is running */
	bool isRunning();

	// Minami's addition
#ifdef macintosh
	virtual int	threadFunction_Mac();
#endif
	// end of addition

protected:
	virtual void loop()=0;

private:
	/** to pause the thread */
	Mutex m_suspend;
	/** to sync actions that might affect each other */
	Mutex m_sync;
	volatile bool m_suspended;
	volatile bool m_terminate;
	volatile bool m_running;
	virtual int threadFunction();
	LoopThread(LoopThread const&);
	LoopThread& operator=(LoopThread const&);
};
#endif // _ARIANNE_THREAD_H
